from horus.resources import UserFactory

def build_routes(config):
    """ Add routes to the config """
    config.add_route('login', '/login')
    config.add_route('logout', '/logout')
    config.add_route('register', '/register')
    config.add_route('activate', '/activate/{user_pk}/{code}', factory=UserFactory)
    config.add_route('forgot_password', '/forgot_password')
    config.add_route('reset_password', '/reset_password/{code}')
    config.add_route('profile', '/profile/{user_pk}', factory=UserFactory,
            traverse="/{user_pk}")
    config.add_route('edit_profile', '/profile/{user_pk}/edit', factory=UserFactory,
            traverse="/{user_pk}")



from zope.interface import Interface

class IHorusUserClass(Interface):
    pass

class IHorusActivationClass(Interface):
    pass

class IHorusSession(Interface):
    pass

class IHorusLoginSchema(Interface):
    pass

class IHorusLoginForm(Interface):
    pass

class IHorusRegisterSchema(Interface):
    pass

class IHorusRegisterForm(Interface):
    pass

class IHorusForgotPasswordForm(Interface):
    pass

class IHorusForgotPasswordSchema(Interface):
    pass

class IHorusResetPasswordForm(Interface):
    pass

class IHorusResetPasswordSchema(Interface):
    pass

class IHorusProfileForm(Interface):
    pass

class IHorusProfileSchema(Interface):
    pass



from pyramid import testing

from mock import Mock
from mock import patch

from horus.tests import UnitTestBase

class TestAuthController(UnitTestBase):
    def test_auth_controller_extensions(self):
        from horus.views        import AuthController
        from horus.interfaces   import IHorusUserClass
        from horus.tests.models import User
        from horus.interfaces   import IHorusLoginSchema
        from horus.interfaces   import IHorusLoginForm
        from horus.interfaces   import IHorusActivationClass
        from horus.tests.models import Activation
        self.config.registry.registerUtility(Activation, IHorusActivationClass)

        self.config.add_route('index', '/')

        request = testing.DummyRequest()

        getUtility = Mock()
        getUtility.return_value = True

        schema = Mock()
        form = Mock()

        self.config.registry.registerUtility(User, IHorusUserClass)
        self.config.registry.registerUtility(schema, IHorusLoginSchema)
        self.config.registry.registerUtility(form, IHorusLoginForm)

        AuthController(request)

        assert schema.called
        assert form.called


    def test_login_loads(self):
        from horus.views          import AuthController
        from horus.interfaces     import IHorusUserClass
        from horus.tests.models   import User
        from horus.interfaces   import IHorusActivationClass
        from horus.tests.models import Activation
        self.config.registry.registerUtility(Activation, IHorusActivationClass)

        self.config.registry.registerUtility(User, IHorusUserClass)
        self.config.add_route('index', '/')
        self.config.include('horus')

        request = testing.DummyRequest()
        request.user = None
        view = AuthController(request)
        response = view.login()

        assert response.get('form', None)

    def test_login_redirects_if_logged_in(self):
        from horus.views import AuthController
        from horus.interfaces     import IHorusUserClass
        from horus.tests.models   import User
        from horus.interfaces   import IHorusActivationClass
        from horus.tests.models import Activation
        self.config.registry.registerUtility(Activation, IHorusActivationClass)

        self.config.registry.registerUtility(User, IHorusUserClass)
        self.config.add_route('index', '/')
        self.config.include('horus')

        request = testing.DummyRequest()
        request.user = Mock()
        view = AuthController(request)

        response = view.login()
        assert response.status_int == 302

    def test_login_fails_empty(self):
        """ Make sure we can't login with empty credentials"""
        from horus.views import AuthController
        from horus.interfaces     import IHorusUserClass
        from horus.tests.models   import User
        from horus.interfaces   import IHorusActivationClass
        from horus.tests.models import Activation
        self.config.registry.registerUtility(Activation, IHorusActivationClass)

        self.config.registry.registerUtility(User, IHorusUserClass)
        self.config.add_route('index', '/')
        self.config.include('horus')

        request = testing.DummyRequest(post={
            'submit': True,
        }, request_method='POST')

        view = AuthController(request)
        response = view.login()
        errors = response['errors']

        assert errors[0].node.name == u'csrf_token'
        assert errors[0].msg == u'Required'
        assert errors[1].node.name == u'User_name'
        assert errors[1].msg == u'Required'
        assert errors[2].node.name == u'Password'
        assert errors[2].msg == u'Required'

    def test_csrf_invalid_fails(self):
        """ Make sure we can't login with a bad csrf """
        from horus.views import AuthController
        from horus.interfaces     import IHorusUserClass
        from horus.tests.models   import User
        from horus.interfaces   import IHorusActivationClass
        from horus.tests.models import Activation
        self.config.registry.registerUtility(Activation, IHorusActivationClass)

        self.config.registry.registerUtility(User, IHorusUserClass)
        self.config.add_route('index', '/')
        self.config.include('horus')

        request = self.get_csrf_request(post={
                    'submit': True,
                    'login': 'admin',
                    'password': 'test123',
                    'csrf_token': 'abc2'
                }, request_method='POST')

        view = AuthController(request)

        response = view.login()

        errors = response['errors']

        assert errors[0].node.name == u'csrf_token'
        assert errors[0].msg == u'Invalid cross-site scripting token'


    def test_login_fails_bad_credentials(self):
        """ Make sure we can't login with bad credentials"""
        from horus.views import AuthController
        from horus.interfaces     import IHorusUserClass
        from horus.tests.models   import User
        from horus.interfaces   import IHorusActivationClass
        from horus.tests.models import Activation
        self.config.registry.registerUtility(Activation, IHorusActivationClass)

        self.config.registry.registerUtility(User, IHorusUserClass)
        self.config.add_route('index', '/')
        self.config.include('horus')

        request = self.get_csrf_request(post={
                'submit': True,
                'User_name': 'admin',
                'Password': 'test123',
            }, request_method='POST')

        flash = Mock()
        request.session.flash = flash

        view = AuthController(request)
        view.login()

        flash.assert_called_with(u'Invalid username or password.', 'error')

    def test_login_succeeds(self):
        """ Make sure we can login """
        from horus.tests.models import User
        from horus.interfaces     import IHorusUserClass
        from horus.interfaces   import IHorusActivationClass
        from horus.tests.models import Activation
        self.config.registry.registerUtility(Activation, IHorusActivationClass)

        self.config.registry.registerUtility(User, IHorusUserClass)

        admin = User(user_name='sontek', email='sontek@gmail.com')
        admin.set_password('foo')

        self.session.add(admin)
        self.session.flush()

        from horus.views import AuthController
        self.config.add_route('index', '/')

        self.config.include('horus')

        request = self.get_csrf_request(post={
                'submit': True,
                'User_name': 'sontek',
                'Password': 'foo',
            }, request_method='POST')

        view = AuthController(request)
        response = view.login()

        assert response.status_int == 302

    def test_inactive_login_fails(self):
        """ Make sure we can't login with an inactive user """
        from horus.tests.models import User
        from horus.interfaces     import IHorusUserClass
        from horus.interfaces   import IHorusActivationClass
        from horus.tests.models import Activation
        self.config.registry.registerUtility(Activation, IHorusActivationClass)

        self.config.registry.registerUtility(User, IHorusUserClass)
        user = User(user_name='sontek', email='sontek@gmail.com')
        user.set_password('foo')
        user.activation = Activation()
        self.session.add(user)
        self.session.flush()

        from horus.views import AuthController
        self.config.add_route('index', '/')
        self.config.include('horus')

        request = self.get_csrf_request(post={
                'submit': True,
                'User_name': 'sontek',
                'Password': 'foo',
            }, request_method='POST')

        flash = Mock()

        request.session.flash = flash

        view = AuthController(request)
        view.login()

        flash.assert_called_with(u'Your account is not active, please check your e-mail.',
            'error')

    def test_logout(self):
        from horus.views        import AuthController
        from horus.tests.models import User
        from horus.interfaces import IHorusUserClass
        from horus.interfaces   import IHorusActivationClass
        from horus.tests.models import Activation
        self.config.registry.registerUtility(Activation, IHorusActivationClass)
        self.config.registry.registerUtility(User, IHorusUserClass)
        self.config.add_route('index', '/')
        self.config.include('horus')
        request = testing.DummyRequest()

        flash = Mock()
        invalidate = Mock()

        request.user = Mock()
        request.session = Mock()
        request.session.invalidate = invalidate
        request.session.flash = flash

        view = AuthController(request)
        with patch('horus.views.forget') as forget:
            with patch('horus.views.HTTPFound') as HTTPFound:
                view.logout()
                flash.assert_called_with(u'Logged out successfully.', 'success')
                forget.assert_called_with(request)
                assert invalidate.called
                assert HTTPFound.called

class TestRegisterController(UnitTestBase):
    def test_register_controller_extensions_with_mail(self):
        from pyramid_mailer.mailer      import DummyMailer
        from pyramid_mailer.interfaces  import IMailer
        from horus.views                import RegisterController
        from horus.interfaces           import IHorusRegisterSchema
        from horus.interfaces           import IHorusRegisterForm
        from horus.interfaces           import IHorusUserClass
        from horus.tests.models         import User
        from horus.interfaces   import IHorusActivationClass
        from horus.tests.models import Activation
        self.config.registry.registerUtility(Activation, IHorusActivationClass)

        self.config.registry.registerUtility(User, IHorusUserClass)

        self.config.add_route('index', '/')

        request = testing.DummyRequest()

        getUtility = Mock()
        getUtility.return_value = True

        schema = Mock()
        form = Mock()

        self.config.registry.registerUtility(schema, IHorusRegisterSchema)
        self.config.registry.registerUtility(form, IHorusRegisterForm)
        self.config.registry.registerUtility(DummyMailer(), IMailer)

        with patch('horus.views.get_mailer') as get_mailer:
            RegisterController(request)
            assert get_mailer.called

        assert schema.called
        assert form.called

    def test_register_controller_extensions_without_mail(self):
        from horus.views import RegisterController
        from horus.interfaces import IHorusRegisterSchema
        from horus.interfaces import IHorusRegisterForm
        from horus.interfaces           import IHorusUserClass
        from horus.tests.models         import User
        from horus.interfaces   import IHorusActivationClass
        from horus.tests.models import Activation
        self.config.registry.registerUtility(Activation, IHorusActivationClass)

        self.config.registry.registerUtility(User, IHorusUserClass)

        self.config.add_route('index', '/')

        request = testing.DummyRequest()

        getUtility = Mock()
        getUtility.return_value = True

        schema = Mock()
        form = Mock()
        
        self.config.registry.settings['su.require_activation'] = False
        self.config.registry.registerUtility(schema, IHorusRegisterSchema)
        self.config.registry.registerUtility(form, IHorusRegisterForm)

        with patch('horus.views.get_mailer') as get_mailer:
            RegisterController(request)
            assert not get_mailer.called

        schema.assert_called_once_with()
        assert form.called

    def test_register_loads_not_logged_in(self):
        from horus.views                import RegisterController
        from pyramid_mailer.mailer      import DummyMailer
        from pyramid_mailer.interfaces  import IMailer
        from horus.interfaces           import IHorusUserClass
        from horus.tests.models         import User
        from horus.interfaces   import IHorusActivationClass
        from horus.tests.models import Activation
        self.config.registry.registerUtility(Activation, IHorusActivationClass)

        self.config.registry.registerUtility(User, IHorusUserClass)
        self.config.registry.registerUtility(DummyMailer(), IMailer)

        self.config.include('horus')

        self.config.add_route('index', '/')

        request = testing.DummyRequest()
        request.user = None
        controller = RegisterController(request)
        response = controller.register()

        assert response.get('form', None)

    def test_register_redirects_if_logged_in(self):
        from horus.views                import RegisterController
        from pyramid_mailer.mailer      import DummyMailer
        from pyramid_mailer.interfaces  import IMailer
        from horus.interfaces           import IHorusUserClass
        from horus.tests.models         import User
        from horus.interfaces   import IHorusActivationClass
        from horus.tests.models import Activation
        self.config.registry.registerUtility(Activation, IHorusActivationClass)

        self.config.registry.registerUtility(User, IHorusUserClass)

        self.config.include('horus')
        self.config.registry.registerUtility(DummyMailer(), IMailer)

        self.config.add_route('index', '/')

        request = testing.DummyRequest()
        request.user = Mock()
        controller = RegisterController(request)
        response = controller.register()

        assert response.status_int == 302

    def test_register_creates_user(self):
        from horus.views import RegisterController
        from pyramid_mailer.mailer import DummyMailer
        from pyramid_mailer.interfaces import IMailer
        from horus.interfaces           import IHorusUserClass
        from horus.tests.models         import User
        from horus.interfaces   import IHorusActivationClass
        from horus.tests.models import Activation
        self.config.registry.registerUtility(Activation, IHorusActivationClass)

        self.config.registry.registerUtility(User, IHorusUserClass)

        self.config.include('horus')
        self.config.registry.registerUtility(DummyMailer(), IMailer)

        self.config.add_route('index', '/')

        request = self.get_csrf_request(post={
            'User_name': 'admin',
            'Password': {
                'Password': 'test123',
                'Password-confirm': 'test123',
            },
            'Email': 'sontek@gmail.com'
        }, request_method='POST')

        request.user = Mock()
        controller = RegisterController(request)
        response = controller.register()

        assert response.status_int == 302

        user = User.get_by_user_name(request, 'admin')

        assert user != None

    def test_register_validation(self):
        from horus.views                import RegisterController
        from pyramid_mailer.mailer      import DummyMailer
        from pyramid_mailer.interfaces  import IMailer
        from horus.interfaces           import IHorusUserClass
        from horus.tests.models         import User
        from horus.interfaces   import IHorusActivationClass
        from horus.tests.models import Activation
        self.config.registry.registerUtility(Activation, IHorusActivationClass)

        self.config.registry.registerUtility(User, IHorusUserClass)

        self.config.include('horus')
        self.config.registry.registerUtility(DummyMailer(), IMailer)

        self.config.add_route('index', '/')

        request = self.get_csrf_request(request_method='POST')

        request.user = Mock()
        controller = RegisterController(request)
        response = controller.register()

        assert len(response['errors']) == 3
        assert 'There was a problem with your submission' in response['form']

    def test_register_existing_user(self):
        from horus.views import RegisterController
        from pyramid_mailer.mailer import DummyMailer
        from pyramid_mailer.interfaces import IMailer
        from horus.interfaces           import IHorusUserClass
        from horus.tests.models         import User
        from horus.interfaces   import IHorusActivationClass
        from horus.tests.models import Activation
        self.config.registry.registerUtility(Activation, IHorusActivationClass)

        self.config.registry.registerUtility(User, IHorusUserClass)

        self.config.include('horus')
        self.config.registry.registerUtility(DummyMailer(), IMailer)

        self.config.add_route('index', '/')

        admin = User(user_name='sontek', email='sontek@gmail.com')
        admin.set_password('test123')
        self.session.add(admin)
        self.session.flush()

        request = self.get_csrf_request(post={
            'User_name': 'sontek',
            'Password': {
                'Password': 'test123',
                'Password-confirm': 'test123',
            },
            'Email': 'sontek@gmail.com'
        }, request_method='POST')

        flash = Mock()
        request.session.flash = flash

        controller = RegisterController(request)
        controller.register()

        flash.assert_called_with(u'That username is already used.', 'error')

    def test_register_no_email_validation(self):
        from horus.views import RegisterController
        from pyramid_mailer.mailer import DummyMailer
        from pyramid_mailer.interfaces import IMailer
        from horus.interfaces import IHorusSession
        from horus.events import NewRegistrationEvent
        from horus.interfaces           import IHorusUserClass
        from horus.tests.models         import User
        from horus.interfaces   import IHorusActivationClass
        from horus.tests.models import Activation
        self.config.registry.registerUtility(Activation, IHorusActivationClass)

        self.config.registry.registerUtility(User, IHorusUserClass)

        self.config.include('horus')
        self.config.registry.registerUtility(DummyMailer(), IMailer)

        self.config.add_route('index', '/')
        self.config.registry.settings['su.require_activation'] = False

        def handle_registration(event):
            request = event.request
            session = request.registry.getUtility(IHorusSession)
            session.commit()

        self.config.add_subscriber(handle_registration, NewRegistrationEvent)


        request = self.get_csrf_request(post={
            'User_name': 'admin',
            'Password': {
                'Password': 'test123',
                'Password-confirm': 'test123',
            },
            'Email': 'sontek@gmail.com'
        }, request_method='POST')

        flash = Mock()
        request.session.flash = flash

        request.user = Mock()

        controller = RegisterController(request)
        response = controller.register()

        assert response.status_int == 302

        user = User.get_by_user_name(request, 'admin')

        assert user.is_activated == True
        flash.assert_called_with('You have been registered, you may login now!', 'success')

    def test_registration_craps_out(self):
        from horus.views                import RegisterController
        from pyramid_mailer.interfaces  import IMailer
        from horus.interfaces           import IHorusUserClass
        from horus.tests.models         import User
        from horus.interfaces   import IHorusActivationClass
        from horus.tests.models import Activation
        self.config.registry.registerUtility(Activation, IHorusActivationClass)

        self.config.registry.registerUtility(User, IHorusUserClass)

        def send(message):
            raise Exception("I broke!")

        mailer = Mock()
        mailer.send = send

        self.config.include('horus')
        self.config.registry.registerUtility(mailer, IMailer)

        self.config.add_route('index', '/')

        request = self.get_csrf_request(post={
            'User_name': 'admin',
            'Password': {
                'Password': 'test123',
                'Password-confirm': 'test123',
            },
            'Email': 'sontek@gmail.com'
        }, request_method='POST')

        flash = Mock()
        request.session.flash = flash

        request.user = Mock()
        controller = RegisterController(request)
        controller.register()

        flash.assert_called_with('I broke!', 'error')

    def test_activate(self):
        from horus.views import RegisterController
        from pyramid_mailer.interfaces import IMailer
        from pyramid_mailer.mailer import DummyMailer
        from horus.interfaces           import IHorusUserClass
        from horus.tests.models         import User
        from horus.interfaces   import IHorusActivationClass
        from horus.tests.models import Activation
        self.config.registry.registerUtility(Activation, IHorusActivationClass)

        self.config.registry.registerUtility(User, IHorusUserClass)
        self.config.include('horus')
        self.config.add_route('index', '/')

        self.config.registry.registerUtility(DummyMailer(), IMailer)

        user = User(user_name='sontek', email='sontek2@gmail.com')
        user.set_password('foo')
        user.activation = Activation()

        self.session.add(user)
        self.session.flush()

        request = testing.DummyRequest()
        request.matchdict = Mock()

        def get(key, default):
            if key == 'code':
                return user.activation.code
            else:
                return user.pk

        request.matchdict.get = get

        controller = RegisterController(request)
        response = controller.activate()
        user = User.get_by_user_name(request, 'sontek')

        assert user.is_activated
        assert response.status_int == 302

    def test_activate_multiple_users(self):
        from horus.views import RegisterController
        from pyramid_mailer.interfaces import IMailer
        from pyramid_mailer.mailer import DummyMailer
        from horus.interfaces           import IHorusUserClass
        from horus.tests.models         import User
        from horus.interfaces           import IHorusActivationClass
        from horus.tests.models         import Activation
        self.config.registry.registerUtility(Activation, IHorusActivationClass)

        self.config.registry.registerUtility(User, IHorusUserClass)
        self.config.include('horus')
        self.config.add_route('index', '/')

        self.config.registry.registerUtility(DummyMailer(), IMailer)

        user = User(user_name='sontek', email='sontek@gmail.com')
        user.activation = Activation()
        user.set_password('foo')
        user1 = User(user_name='sontek1', email='sontek+2@gmail.com')
        user1.activation = Activation()
        user1.set_password('foo2')

        self.session.add(user)
        self.session.add(user1)
        self.session.flush()

        request = testing.DummyRequest()
        request.matchdict = Mock()

        def get(key, default):
            if key == 'code':
                return user1.activation.code
            else:
                return user1.pk

        request.matchdict.get = get

        controller = RegisterController(request)
        response = controller.activate()
        user = User.get_by_user_name(request, 'sontek1')

        activations = Activation.get_all(request)

        assert len(activations) == 1
        assert user.is_activated
        assert response.status_int == 302

    def test_activate_invalid(self):
        from horus.views import RegisterController
        from pyramid_mailer.interfaces import IMailer
        from pyramid_mailer.mailer import DummyMailer
        from horus.interfaces           import IHorusUserClass
        from horus.tests.models         import User
        from horus.interfaces           import IHorusActivationClass
        from horus.tests.models         import Activation
        self.config.registry.registerUtility(Activation, IHorusActivationClass)

        self.config.registry.registerUtility(User, IHorusUserClass)
        self.config.include('horus')
        self.config.add_route('index', '/')

        self.config.registry.registerUtility(DummyMailer(), IMailer)

        user = User(user_name='sontek', email='sontek@gmail.com')
        user.set_password('temp')
        user.activation = Activation()

        self.session.add(user)
        self.session.flush()

        request = testing.DummyRequest()
        request.matchdict = Mock()
        get = Mock()
        get.return_value = 'invalid'
        request.matchdict.get = get

        controller = RegisterController(request)
        response = controller.activate()
        user = User.get_by_user_name(request, 'sontek')

        assert not user.is_activated
        assert response.status_int == 404

    def test_activate_invalid_user(self):
        from horus.views import RegisterController
        from pyramid_mailer.interfaces import IMailer
        from pyramid_mailer.mailer import DummyMailer
        from horus.interfaces           import IHorusUserClass
        from horus.tests.models         import User
        from horus.interfaces           import IHorusActivationClass
        from horus.tests.models         import Activation
        self.config.registry.registerUtility(Activation, IHorusActivationClass)

        self.config.registry.registerUtility(User, IHorusUserClass)
        self.config.include('horus')
        self.config.add_route('index', '/')

        self.config.registry.registerUtility(DummyMailer(), IMailer)

        bad_act = Activation()

        user = User(user_name='sontek', email='sontek@gmail.com')
        user.activation = Activation()
        user.set_password('foo')

        user2 = User(user_name='jessie', email='sontek+2@gmail.com')
        user2.activation = bad_act
        user2.set_password('foo2')

        self.session.add(user)
        self.session.add(user2)
        self.session.flush()

        request = testing.DummyRequest()
        request.matchdict = Mock()

        def get(val, ret):
            if val == 'code':
                return bad_act.code
            elif val == 'user_pk':
                return user.pk

        request.matchdict.get = get

        controller = RegisterController(request)
        response = controller.activate()
        new_user1 = User.get_by_user_name(request, 'sontek')
        new_user2 = User.get_by_user_name(request, 'jessie')

        assert not new_user1.is_activated
        assert not new_user2.is_activated
        assert response.status_int == 404

class TestForgotPasswordController(UnitTestBase):
    def test_forgot_password_loads(self):
        from horus.views import ForgotPasswordController
        from horus.interfaces           import IHorusUserClass
        from horus.tests.models         import User
        from horus.interfaces           import IHorusActivationClass
        from horus.tests.models         import Activation
        self.config.registry.registerUtility(Activation, IHorusActivationClass)

        self.config.registry.registerUtility(User, IHorusUserClass)
        self.config.add_route('index', '/')
        self.config.include('horus')

        request = testing.DummyRequest()
        request.user = None
        view = ForgotPasswordController(request)
        response = view.forgot_password()

        assert response.get('form', None)

    def test_forgot_password_logged_in_redirects(self):
        from horus.views import ForgotPasswordController
        from horus.interfaces           import IHorusUserClass
        from horus.tests.models         import User

        self.config.registry.registerUtility(User, IHorusUserClass)
        self.config.add_route('index', '/')
        self.config.include('horus')

        request = testing.DummyRequest()
        request.user = Mock()
        view = ForgotPasswordController(request)
        response = view.forgot_password()

        assert response.status_int == 302

    def test_forgot_password_valid_user(self):
        from horus.views import ForgotPasswordController
        from pyramid_mailer.interfaces import IMailer
        from pyramid_mailer.mailer import DummyMailer
        from horus.interfaces           import IHorusUserClass
        from horus.tests.models         import User

        self.config.registry.registerUtility(User, IHorusUserClass)

        self.config.add_route('index', '/')
        self.config.include('horus')
        self.config.registry.registerUtility(DummyMailer(), IMailer)

        user = User(user_name='sontek', password='temp', email='sontek@gmail.com')
        user.set_password('foo')

        self.session.add(user)
        self.session.flush()


        request = self.get_csrf_request(post={
            'Email': 'sontek@gmail.com'
        }, request_method='POST')

        request.user = None

        flash = Mock()
        request.session.flash = flash

        view = ForgotPasswordController(request)
        response = view.forgot_password()

        flash.assert_called_with(u'Please check your e-mail to reset your password.', 'success')
        assert response.status_int == 302

    def test_forgot_password_invalid_password(self):
        from horus.views import ForgotPasswordController
        from pyramid_mailer.interfaces import IMailer
        from pyramid_mailer.mailer import DummyMailer
        from horus.interfaces           import IHorusUserClass
        from horus.tests.models         import User

        self.config.registry.registerUtility(User, IHorusUserClass)

        self.config.add_route('index', '/')
        self.config.include('horus')
        self.config.registry.registerUtility(DummyMailer(), IMailer)

        user = User(user_name='sontek', password='temp', email='sontek@gmail.com')
        user.set_password('foo')

        self.session.add(user)
        self.session.flush()

        request = self.get_csrf_request(post={
            'Email': 'sontek'
        }, request_method='POST')

        request.user = None

        view = ForgotPasswordController(request)
        response = view.forgot_password()

        assert len(response['errors']) == 1

    def test_reset_password_loads(self):
        from horus.views import ForgotPasswordController
        from pyramid_mailer.interfaces import IMailer
        from pyramid_mailer.mailer import DummyMailer
        from horus.interfaces           import IHorusUserClass
        from horus.tests.models         import User
        from horus.tests.models         import Activation
        from horus.interfaces           import IHorusActivationClass

        self.config.registry.registerUtility(User, IHorusUserClass)
        self.config.registry.registerUtility(Activation, IHorusActivationClass)

        self.config.add_route('index', '/')
        self.config.include('horus')
        self.config.registry.registerUtility(DummyMailer(), IMailer)

        user = User(user_name='sontek', password='temp', email='sontek@gmail.com')
        user.set_password('foo')
        user.activation = Activation()

        self.session.add(user)
        self.session.flush()

        request = testing.DummyRequest()

        request.matchdict = Mock()
        get = Mock()
        get.return_value = user.activation.code
        request.matchdict.get = get

        request.user = None

        view = ForgotPasswordController(request)
        response = view.reset_password()

        assert response.get('form', None)
        assert 'sontek' in response['form']

    def test_reset_password_valid_user(self):
        from horus.views import ForgotPasswordController
        from horus.interfaces import IHorusSession
        from horus.events import PasswordResetEvent
        from pyramid_mailer.interfaces import IMailer
        from pyramid_mailer.mailer import DummyMailer
        from horus.models import crypt
        from horus.interfaces           import IHorusUserClass
        from horus.interfaces           import IHorusActivationClass
        from horus.tests.models         import User
        from horus.tests.models         import Activation

        self.config.registry.registerUtility(User, IHorusUserClass)
        self.config.registry.registerUtility(Activation, IHorusActivationClass)


        self.config.add_route('index', '/')
        self.config.include('horus')
        self.config.registry.registerUtility(DummyMailer(), IMailer)

        user = User(user_name='sontek', email='sontek@gmail.com')
        user.set_password('foo')
        user.activation = Activation()

        self.session.add(user)
        self.session.flush()

        request = self.get_csrf_request(post={
            'Password': {
                'Password': 'test123',
                'Password-confirm': 'test123',
            },
        }, request_method='POST')

        request.matchdict = Mock()
        get = Mock()
        get.return_value = user.activation.code
        request.matchdict.get = get

        request.user = None

        flash = Mock()
        request.session.flash = flash

        def handle_password_reset(event):
            request = event.request
            session = request.registry.getUtility(IHorusSession)
            session.commit()

        self.config.add_subscriber(handle_password_reset, PasswordResetEvent)

        view = ForgotPasswordController(request)
        response = view.reset_password()

        assert not crypt.check(user.password, 'temp' + user.salt)
        assert response.status_int == 302

    def test_reset_password_invalid_password(self):
        from horus.views import ForgotPasswordController
        from pyramid_mailer.interfaces import IMailer
        from pyramid_mailer.mailer import DummyMailer
        from horus.interfaces           import IHorusUserClass
        from horus.interfaces           import IHorusActivationClass
        from horus.tests.models         import User
        from horus.tests.models         import Activation

        self.config.registry.registerUtility(User, IHorusUserClass)
        self.config.registry.registerUtility(Activation, IHorusActivationClass)

        self.config.add_route('index', '/')
        self.config.include('horus')
        self.config.registry.registerUtility(DummyMailer(), IMailer)


        user = User(user_name='sontek', password='temp', email='sontek@gmail.com')
        user.set_password('foo')
        user.activation = Activation()

        self.session.add(user)
        self.session.flush()

        request = self.get_csrf_request(post={
            'Password': {
                'Password': 't',
                'Password-confirm': 't',
            },
        }, request_method='POST')

        request.matchdict = Mock()
        get = Mock()
        get.return_value = user.activation.code
        request.matchdict.get = get

        request.user = None

        flash = Mock()
        request.session.flash = flash

        view = ForgotPasswordController(request)
        response = view.reset_password()

        assert len(response['errors']) == 1

    def test_reset_password_empty_password(self):
        from horus.views import ForgotPasswordController
        from pyramid_mailer.interfaces import IMailer
        from pyramid_mailer.mailer import DummyMailer
        from horus.interfaces           import IHorusUserClass
        from horus.interfaces           import IHorusActivationClass
        from horus.tests.models         import User
        from horus.tests.models         import Activation

        self.config.registry.registerUtility(User, IHorusUserClass)
        self.config.registry.registerUtility(Activation, IHorusActivationClass)

        self.config.add_route('index', '/')
        self.config.include('horus')
        self.config.registry.registerUtility(DummyMailer(), IMailer)


        user = User(user_name='sontek', password='temp', email='sontek@gmail.com')
        user.set_password('foo')
        user.activation = Activation()

        self.session.add(user)
        self.session.flush()

        request = self.get_csrf_request(request_method='POST')

        request.matchdict = Mock()
        get = Mock()
        get.return_value = user.activation.code
        request.matchdict.get = get

        request.user = None

        flash = Mock()
        request.session.flash = flash

        view = ForgotPasswordController(request)

        response = view.reset_password()

        assert len(response['errors']) == 1

    def test_invalid_reset_gets_404(self):
        from horus.views                import ForgotPasswordController
        from pyramid_mailer.interfaces  import IMailer
        from pyramid_mailer.mailer      import DummyMailer
        from horus.interfaces           import IHorusUserClass
        from horus.interfaces           import IHorusActivationClass
        from horus.tests.models         import User
        from horus.tests.models         import Activation

        self.config.registry.registerUtility(User, IHorusUserClass)
        self.config.registry.registerUtility(Activation, IHorusActivationClass)

        self.config.add_route('index', '/')
        self.config.include('horus')
        self.config.registry.registerUtility(DummyMailer(), IMailer)

        user = User(user_name='sontek', password='temp', email='sontek@gmail.com')
        user.set_password('foo')
        user.activation = Activation()

        self.session.add(user)
        self.session.flush()

        request = testing.DummyRequest()

        request.matchdict = Mock()
        get = Mock()
        get.return_value = 'b'
        request.matchdict.get = get

        request.user = None

        view = ForgotPasswordController(request)
        response = view.reset_password()

        assert response.status_int == 404

class TestProfileController(UnitTestBase):
    def test_profile_loads(self):
        from horus.views import ProfileController
        from horus.interfaces           import IHorusUserClass
        from horus.interfaces           import IHorusActivationClass
        from horus.tests.models         import User
        from horus.tests.models         import Activation

        self.config.registry.registerUtility(User, IHorusUserClass)
        self.config.registry.registerUtility(Activation, IHorusActivationClass)

        self.config.add_route('index', '/')
        self.config.include('horus')

        user = User(user_name='sontek',
                email='sontek@gmail.com')
        user.set_password('temp')
        self.session.add(user)
        self.session.flush()

        request = testing.DummyRequest()
        request.user = Mock()

        flash = Mock()
        request.session.flash = flash

        request.matchdict = Mock()
        get = Mock()
        get.return_value = user.pk
        request.matchdict.get = get


        view = ProfileController(request)

        response = view.profile()

        assert response.get('user', None) == user

    def test_profile_bad_pk(self):
        from horus.views import ProfileController
        from horus.interfaces           import IHorusUserClass
        from horus.interfaces           import IHorusActivationClass
        from horus.tests.models         import User
        from horus.tests.models         import Activation

        self.config.registry.registerUtility(User, IHorusUserClass)
        self.config.registry.registerUtility(Activation, IHorusActivationClass)

        self.config.add_route('index', '/')
        self.config.include('horus')

        user = User(user_name='sontek', email='sontek@gmail.com')
        user.set_password('temp')

        self.session.add(user)
        self.session.flush()

        request = testing.DummyRequest()
        request.user = Mock()

        flash = Mock()
        request.session.flash = flash

        request.matchdict = Mock()
        get = Mock()
        get.return_value = 99
        request.matchdict.get = get

        view = ProfileController(request)

        response = view.profile()

        assert response.status_int == 404

    def test_profile_update_profile_invalid(self):
        from horus.views import ProfileController
        from horus.interfaces           import IHorusUserClass
        from horus.interfaces           import IHorusActivationClass
        from horus.interfaces           import IHorusProfileSchema
        from horus.tests.models         import User
        from horus.tests.models         import Activation
        from horus.tests.schemas        import ProfileSchema

        self.config.registry.registerUtility(Activation, IHorusActivationClass)
        self.config.registry.registerUtility(User, IHorusUserClass)
        self.config.registry.registerUtility(ProfileSchema,
            IHorusProfileSchema)

        self.config.add_route('index', '/')
        self.config.include('horus')

        user = User(user_name='sontek', email='sontek@gmail.com')
        user.set_password('temp')
        self.session.add(user)
        self.session.flush()

        request = self.get_csrf_request(request_method='POST') 
        request.context = user

        request.matchdict = Mock()
        get = Mock()
        get.return_value = user.pk
        request.matchdict.get = get


        flash = Mock()
        request.session.flash = flash

        view = ProfileController(request)

        response = view.edit_profile()

        assert len(response['errors']) == 3

    def test_profile_update_profile(self):
        from horus.views import ProfileController
        from horus.interfaces import IHorusSession
        from horus.events import ProfileUpdatedEvent
        from horus.models import crypt
        from horus.interfaces           import IHorusUserClass
        from horus.tests.models         import User
        from horus.interfaces           import IHorusActivationClass
        from horus.tests.models         import Activation
        self.config.registry.registerUtility(Activation, IHorusActivationClass)

        self.config.registry.registerUtility(User, IHorusUserClass)

        self.config.add_route('index', '/')
        self.config.include('horus')

        user = User(user_name='sontek', email='sontek@gmail.com')
        user.set_password('temp')
        self.session.add(user)
        self.session.flush()

        def handle_profile_updated(event):
            request = event.request
            session = request.registry.getUtility(IHorusSession)
            session.commit()

        self.config.add_subscriber(handle_profile_updated, ProfileUpdatedEvent)

        request = self.get_csrf_request(post={
            'Email': 'sontek@gmail.com',
        }, request_method='POST')

        request.user = user

        flash = Mock()
        request.session.flash = flash

        request.matchdict = Mock()
        get = Mock()
        get.return_value = user.pk
        request.matchdict.get = get

        view = ProfileController(request)
        view.profile()

        new_user = User.get_by_pk(request, user.pk)

        assert new_user.email == 'sontek@gmail.com'
        assert crypt.check(user.password, 'temp' + user.salt)

    def test_profile_update_password(self):
        from horus.views import ProfileController
        from horus.interfaces import IHorusSession
        from horus.events import ProfileUpdatedEvent
        from horus.models import crypt
        from horus.interfaces           import IHorusUserClass
        from horus.tests.models         import User
        from horus.interfaces           import IHorusActivationClass
        from horus.tests.models         import Activation
        self.config.registry.registerUtility(Activation, IHorusActivationClass)

        self.config.registry.registerUtility(User, IHorusUserClass)

        self.config.add_route('index', '/')
        self.config.include('horus')

        user = User(user_name='sontek', email='sontek@gmail.com')
        user.set_password('temp')

        self.session.add(user)
        self.session.flush()

        def handle_profile_updated(event):
            request = event.request
            session = request.registry.getUtility(IHorusSession)
            session.commit()

        self.config.add_subscriber(handle_profile_updated, ProfileUpdatedEvent)


        request = self.get_csrf_request(post={
            'Email': 'sontek@gmail.com',
            'Password': {
                'Password': 'test123',
                'Password-confirm': 'test123',
            },
        }, request_method='POST')

        request.context = user

        request.matchdict = Mock()
        get = Mock()
        get.return_value = user.pk
        request.matchdict.get = get

        flash = Mock()
        request.session.flash = flash

        view = ProfileController(request)

        view.edit_profile()
        new_user = User.get_by_pk(request, user.pk)

        assert new_user.email == 'sontek@gmail.com'
        assert not crypt.check(user.password, 'temp' + user.salt)



from pyramid.i18n               import TranslationStringFactory
from pyramid.security           import Allow
from datetime                   import datetime
from datetime                   import timedelta
from datetime                   import date
from sqlalchemy.ext.declarative import declared_attr
from sqlalchemy                 import or_
from sqlalchemy                 import func

from horus.lib                  import generate_random_string
from horus.lib                  import get_session
from horus.lib                  import pluralize

import cryptacular.bcrypt
import re
import urllib
import hashlib
import sqlalchemy as sa


_ = TranslationStringFactory('horus')

crypt = cryptacular.bcrypt.BCRYPTPasswordManager()

class BaseModel(object):
    """Base class which auto-generates tablename, and surrogate
    primary key column.
    """
    __table_args__ = {
        'sqlite_autoincrement': True,
        'mysql_engine': 'InnoDB',
        'mysql_charset': 'utf8'
    }

    @declared_attr
    def __tablename__(cls):
        """Convert CamelCase class name to underscores_between_words 
        table name."""
        name = cls.__name__.replace('Mixin', '')

        return (
            name[0].lower() + 
            re.sub(r'([A-Z])', lambda m:"_" + m.group(0).lower(), name[1:])
        )

    # We use pk instead of id because id is a python builtin
    pk =  sa.Column(sa.Integer, autoincrement=True, primary_key=True)

    def __json__(self):
        """Converts all the properties of the object into a dict
        for use in json
        """
        props = {}

        for key in self.__dict__:
            if not key.startswith('__') and not key.startswith('_sa_'):
                obj = getattr(self, key)

                if isinstance(obj, datetime) or isinstance(obj, date):
                        props[key] = obj.isoformat()
                else:
                    props[key] = getattr(self, key)

        return props

    @classmethod
    def get_all(cls, request):
        session = get_session(request)

        return session.query(cls).all()

    @classmethod
    def get_by_pk(cls, request, pk):
        """Gets an object by its primary key"""
        session = get_session(request)

        return session.query(cls).filter(cls.pk == pk).first()


class ActivationMixin(BaseModel):
    """
    Handle activations/password reset items for users

    The code should be a random hash that is valid only one time
    After that hash is used to access the site it'll be removed

    The created by is a system: new user registration, password reset, forgot
    password, etc.

    """
    @declared_attr
    def code(self):
        """ Unique user name """
        return sa.Column(sa.Unicode(30), nullable=False, unique=True)

    @declared_attr
    def valid_until(self):
        """ How long will the activation key last """
        return sa.Column(sa.DateTime, nullable=False)

    @declared_attr
    def created_by(self):
        """ The system that generated the activation key """
        return sa.Column(sa.Unicode(30), nullable=False)

    @classmethod
    def get_by_code(cls, request, code):
        session = get_session(request)
        return session.query(cls).filter(cls.code == code).first()

    def __init__(self, created_by='web', valid_until=None):
        """ Create a new activation, valid_until is a datetime, 
        defaults to 3 days from current day
        """
        self.code =  generate_random_string(12)
        self.created_by = created_by

        if valid_until:
            self.valid_until = valid_until
        else:
             self.valid_until = datetime.utcnow() + timedelta(days=3)

class UserMixin(BaseModel):
    @declared_attr
    def user_name(self):
        """ Unique user name """
        return sa.Column(sa.Unicode(30), nullable=False, unique=True)

    @declared_attr
    def email(self):
        """ E-mail for user """
        return sa.Column(sa.Unicode(100), nullable=False, unique=True)

    @declared_attr
    def status(self):
        """ Status of user """
        return sa.Column(sa.Integer())

    @declared_attr
    def security_code(self):
        """ Security code user, can be used for API calls or password reset """
        return sa.Column(sa.Unicode(256), nullable=True)

    @declared_attr
    def last_login_date(self):
        """ Date of user's last login """
        return sa.Column(
            sa.TIMESTAMP(timezone=False)
            , default=sa.func.now()
            , server_default=sa.func.now()
            , nullable=False
        )

    @declared_attr
    def registered_date(self):
        """ Date of user's registration """
        return sa.Column(
            sa.TIMESTAMP(timezone=False)
            , default=sa.sql.func.now()
            , server_default=sa.func.now()
            , nullable=False
        )

    @declared_attr
    def salt(self):
        """ Password salt for user """
        return sa.Column(sa.Unicode(256), nullable=False)

    @declared_attr
    def password(self):
        """ Password hash for user object """
        return sa.Column(sa.Unicode(256), nullable=False)

    @declared_attr
    def activation_pk(self):
        return sa.Column(
            sa.Integer,
            sa.ForeignKey('%s.pk' % ActivationMixin.__tablename__)
        )

    @declared_attr
    def activation(self):
        return sa.orm.relationship(
            'Activation',
            backref='user'
        )

    @property
    def is_activated(self):
        return self.activation_pk == None

    def set_password(self, raw_password):
        self.password = self.hash_password(raw_password)

    def hash_password(self, password):
        if not self.salt:
            self.salt = generate_random_string(24)

        return unicode(crypt.encode(password + self.salt))

    def gravatar_url(self, default='mm'):
        """ returns user gravatar url """
        # construct the url
        h = hashlib.md5(self.email.encode('utf8').lower()).hexdigest()
        base_url = "https://secure.gravatar.com/avatar/%s?%s"
        gravatar_url = base_url % (h, urllib.urlencode({'d': default}))

        return gravatar_url

    @classmethod
    def generate_random_password(cls, chars=12):
        """ generates random string of fixed length"""
        return generate_random_string(chars)


    @classmethod
    def get_by_email(cls, request, email):
        session = get_session(request)

        return session.query(cls).filter(
                func.lower(cls.email) == email.lower()
        ).first()

    @classmethod
    def get_by_user_name(cls, request, user_name):
        session = get_session(request)

        return session.query(cls).filter(
            func.lower(cls.user_name) == user_name.lower(),
        ).first()

    @classmethod
    def get_by_user_name_or_email(cls, request, user_name, email):
        session = get_session(request)

        return session.query(cls).filter(
            or_(
                func.lower(cls.user_name) == user_name.lower(),
                cls.email == email
            )
        ).first()

    @classmethod
    def get_by_email_password(cls, request, email, password):
        user = cls.get_by_email(request, email)

        if user:
            valid = cls.validate_user(request, user, password)

            if valid:
                return user

    @classmethod
    def get_by_activation(cls, request, activation):
        session = get_session(request)
        user = session.query(cls).filter(cls.activation_pk == activation.pk).first()
        return user

    @classmethod
    def get_user(cls, request, user_name, password):
        user = cls.get_by_user_name(request, user_name)

        valid = cls.validate_user(user, password)

        if valid:
            return user

    @classmethod
    def validate_user(cls, user, password):
        if not user:
            return None

        if user.password == None:
            valid = False
        else:
            valid = crypt.check(user.password, password + user.salt)

        return valid

    def __repr__(self):
        return '<User: %s>' % self.user_name

    @property
    def __acl__(self):
        return [
                (Allow, 'user:%s' % self.pk, 'access_user')
        ]

class GroupMixin(BaseModel):
    """ base mixin for group object"""

    @declared_attr
    def name(self):
        return sa.Column(sa.Unicode(50), unique=True)

    @declared_attr
    def description(self):
        return sa.Column(sa.UnicodeText())

    @declared_attr
    def users(self):
        """ relationship for users belonging to this group"""
        return sa.orm.relationship(
            'User'
            , secondary=UserGroupMixin.__tablename__
#            , order_by='%s.user.user_name' % UserMixin.__tablename__
            , passive_deletes=True
            , passive_updates=True
            , backref=pluralize(GroupMixin.__tablename__)
        )

#    @declared_attr
#    def permissions(self):
#        """ permissions assigned to this group"""
#        return sa.orm.relationship('GroupPermission'
#            , backref='groups'
#            , cascade="all, delete-orphan"
#            , passive_deletes=True
#            , passive_updates=True
#        )

    def __repr__(self):
        return '<Group: %s>' % self.name

class UserGroupMixin(BaseModel):
    @declared_attr
    def group_pk(self):
        return sa.Column(sa.Integer,
            sa.ForeignKey('%s.pk' % GroupMixin.__tablename__)
        )

    @declared_attr
    def user_pk(self):
        return sa.Column(
            sa.Integer
            , sa.ForeignKey('%s.pk' % UserMixin.__tablename__,
                onupdate='CASCADE',
                ondelete='CASCADE'
            )
            , primary_key=True
        )

    def __repr__(self):
        return '<UserGroup: %s, %s>' % (self.group_name, self.user_pk,)



from horus.tests import UnitTestBase
from pyramid import testing

from mock import Mock

class TestLib(UnitTestBase):
    def test_get_session(self):
        from horus.interfaces import IHorusSession
        from horus.lib import get_session
        request = testing.DummyRequest()
        request.registry = Mock()

        session = Mock()

        getUtility = Mock()
        getUtility.return_value = session

        request.registry.getUtility = getUtility

        new_session = get_session(request)

        getUtility.assert_called_with(IHorusSession)
        assert new_session == session





from pyramid.security   import unauthenticated_userid
from horus.interfaces   import IHorusSession
from horus.interfaces   import IHorusUserClass

import hashlib
import random
import string

def generate_random_string(length):
    """Generate a generic hash key for the user to use"""
    m = hashlib.sha256()
    word = ''

    for i in xrange(length):
        word += random.choice(string.ascii_letters)

    m.update(word)

    return unicode(m.hexdigest()[:length])

def get_session(request):
    session = request.registry.getUtility(IHorusSession)

    return session

def get_user(request):
    pk = unauthenticated_userid(request)
    user_class = request.registry.queryUtility(IHorusUserClass)

    if pk is not None:
        return user_class.get_by_pk(request, pk)

def get_class_from_config(settings, key):
    if key in settings:
        user_modules = settings.get(key).split('.')
        module = '.'.join(user_modules[:-1])
        klass = user_modules[-1]
        imported_module = __import__(module, fromlist=[klass])
        imported_class = getattr(imported_module, klass)

        return imported_class
    else:
        raise Exception('Please provide a horus.userclass config option')


def pluralize(singular):
    """Return plural form of given lowercase singular word (English only). Based on
    ActiveState recipe http://code.activestate.com/recipes/413172/

    >>> pluralize('')
    ''
    >>> pluralize('goose')
    'geese'
    >>> pluralize('dolly')
    'dollies'
    >>> pluralize('genius')
    'genii'
    >>> pluralize('jones')
    'joneses'
    >>> pluralize('pass')
    'passes'
    >>> pluralize('zero')
    'zeros'
    >>> pluralize('casino')
    'casinos'
    >>> pluralize('hero')
    'heroes'
    >>> pluralize('church')
    'churches'
    >>> pluralize('x')
    'xs'
    >>> pluralize('car')
    'cars'

    """
    ABERRANT_PLURAL_MAP = {
        'appendix': 'appendices',
        'barracks': 'barracks',
        'cactus': 'cacti',
        'child': 'children',
        'criterion': 'criteria',
        'deer': 'deer',
        'echo': 'echoes',
        'elf': 'elves',
        'embargo': 'embargoes',
        'focus': 'foci',
        'fungus': 'fungi',
        'goose': 'geese',
        'hero': 'heroes',
        'hoof': 'hooves',
        'index': 'indices',
        'knife': 'knives',
        'leaf': 'leaves',
        'life': 'lives',
        'man': 'men',
        'mouse': 'mice',
        'nucleus': 'nuclei',
        'person': 'people',
        'phenomenon': 'phenomena',
        'potato': 'potatoes',
        'self': 'selves',
        'syllabus': 'syllabi',
        'tomato': 'tomatoes',
        'torpedo': 'torpedoes',
        'veto': 'vetoes',
        'woman': 'women',
        }

    VOWELS = set('aeiou')

    if not singular:
        return ''
    plural = ABERRANT_PLURAL_MAP.get(singular)
    if plural:
        return plural
    root = singular
    try:
        if singular[-1] == 'y' and singular[-2] not in VOWELS:
            root = singular[:-1]
            suffix = 'ies'
        elif singular[-1] == 's':
            if singular[-2] in VOWELS:
                if singular[-3:] == 'ius':
                    root = singular[:-2]
                    suffix = 'i'
                else:
                    root = singular[:-1]
                    suffix = 'ses'
            else:
                suffix = 'es'
        elif singular[-2:] in ('ch', 'sh'):
            suffix = 'es'
        else:
            suffix = 's'
    except IndexError:
        suffix = 's'
    plural = root + suffix
    return plural



#from horus.tests import UnitTestBase
#from pyramid import testing
#
#class TestUserManager(UnitTestBase):
#    def test_get_valid_user(self):
#        from horus.models import User
#        from horus.managers import UserManager
#
#        user = User(username='sontek', password='temp')
#        self.session.add(user)
#        self.session.commit()
#
#        request = testing.DummyRequest()
#        mgr = UserManager(request)
#
#        new_user = mgr.get_user('sontek', 'temp')
#
#        assert user == new_user
#
#    def test_get_all_users(self):
#        from horus.models import User
#        from horus.managers import UserManager
#
#        user = User(username='sontek', password='temp')
#        user2 = User(username='sontek2', password='temp')
#        self.session.add(user)
#        self.session.add(user2)
#        self.session.commit()
#
#        request = testing.DummyRequest()
#        mgr = UserManager(request)
#
#        users = mgr.get_all()
#
#        assert len(users) == 2
#
#    def test_get_invalid_user(self):
#        from horus.models import User
#        from horus.managers import UserManager
#
#        user = User(username='sontek1', password='temp')
#        self.session.add(user)
#        self.session.commit()
#
#        request = testing.DummyRequest()
#        mgr = UserManager(request)
#
#        new_user = mgr.get_user('sontek', 'temp')
#
#        assert new_user == None
#
#    def test_get_user_by_pk(self):
#        from horus.models import User
#        from horus.managers import UserManager
#
#        user = User(username='sontek', password='temp')
#        self.session.add(user)
#        self.session.commit()
#
#        request = testing.DummyRequest()
#        mgr = UserManager(request)
#
#        new_user = mgr.get_by_pk(user.pk)
#
#        assert new_user == user
#
#    def test_get_user_by_invalid_pk(self):
#        from horus.models import User
#        from horus.managers import UserManager
#
#        user = User(username='sontek', password='temp')
#        self.session.add(user)
#        self.session.commit()
#
#        request = testing.DummyRequest()
#        mgr = UserManager(request)
#
#        new_user = mgr.get_by_pk(2)
#
#        assert new_user == None
#
#    def test_get_user_by_username(self):
#        from horus.models import User
#        from horus.managers import UserManager
#
#        user = User(username='sontek', password='temp')
#        self.session.add(user)
#        self.session.commit()
#
#        request = testing.DummyRequest()
#        mgr = UserManager(request)
#
#        new_user = mgr.get_by_username('sontek')
#
#        assert new_user == user
#
#    def test_get_user_by_invalid_username(self):
#        from horus.models import User
#        from horus.managers import UserManager
#
#        user = User(username='sontek', password='temp')
#        self.session.add(user)
#        self.session.commit()
#
#        request = testing.DummyRequest()
#        mgr = UserManager(request)
#
#        new_user = mgr.get_by_username('sontek1')
#
#        assert new_user == None
#
#    def test_get_user_by_email(self):
#        from horus.models import User
#        from horus.managers import UserManager
#
#        user = User(username='sontek', password='temp', email='sontek@gmail.com')
#        self.session.add(user)
#        self.session.commit()
#
#        request = testing.DummyRequest()
#        mgr = UserManager(request)
#
#        new_user = mgr.get_by_email(user.email)
#
#        assert new_user == user
#
#    def test_get_user_by_invalid_email(self):
#        from horus.models import User
#        from horus.managers import UserManager
#
#        user = User(username='sontek', password='temp', email='sontek@gmail.com')
#        self.session.add(user)
#        self.session.commit()
#
#        request = testing.DummyRequest()
#        mgr = UserManager(request)
#
#        new_user = mgr.get_by_email('sontek1@gmail.com')
#
#        assert new_user == None
#
#    def test_get_user_by_activation(self):
#        from horus.models import User
#        from horus.models import Activation
#        from horus.managers import UserManager
#
#        user = User(username='sontek', password='temp', email='sontek@gmail.com')
#        activation = Activation()
#        user.activation = activation
#
#        self.session.add(user)
#        self.session.commit()
#
#        request = testing.DummyRequest()
#        mgr = UserManager(request)
#
#        new_user = mgr.get_by_activation(activation)
#
#        assert new_user == user
#
#    def test_get_user_by_activation_with_multiple_users(self):
#        from horus.models import User
#        from horus.models import Activation
#        from horus.managers import UserManager
#
#        user1 = User(username='sontek1', password='temp', email='sontek@gmail.com')
#        user2 = User(username='sontek2', password='temp', email='sontek@gmail.com')
#
#        activation = Activation()
#        user2.activation = activation
#
#        self.session.add(user1)
#        self.session.add(user2)
#
#        self.session.commit()
#
#        request = testing.DummyRequest()
#        mgr = UserManager(request)
#
#        new_user = mgr.get_by_activation(activation)
#
#        assert new_user == user2
#
#class TestActivationManager(UnitTestBase):
#    def test_get_activation(self):
#        from horus.models import Activation
#        from horus.managers import ActivationManager
#
#        activation = Activation()
#        self.session.add(activation)
#        self.session.commit()
#
#        request = testing.DummyRequest()
#        mgr = ActivationManager(request)
#
#        new_activation = mgr.get_by_code(activation.code)
#
#        assert activation == new_activation
#
#    def test_get_user_activation(self):
#        from horus.models import Activation
#        from horus.managers import ActivationManager
#        from horus.managers import UserManager
#        from horus.models import User
#
#        user1 = User(username='sontek1', password='temp', email='sontek@gmail.com')
#        user2 = User(username='sontek2', password='temp', email='sontek@gmail.com')
#
#        activation = Activation()
#        user2.activation = activation
#
#        self.session.add(user1)
#        self.session.add(user2)
#        self.session.commit()
#
#        request = testing.DummyRequest()
#        mgr = ActivationManager(request)
#        user_mgr = UserManager(request)
#
#        new_user = user_mgr.get_by_username('sontek2')
#
#        new_activation = mgr.get_by_code(activation.code)
#
#        assert activation == new_activation
#        assert new_user.activation == new_activation
#
#class TestOrganizationManager(UnitTestBase):
#    def test_get_organization(self):
#        from horus.models import Organization
#        from horus.models import User
#        from horus.managers import OrganizationManager
#
#        user = User(username='sontek', password='temp')
#        self.session.add(user)
#
#        organization = Organization('test org', user)
#        self.session.add(organization)
#        self.session.commit()
#
#        request = testing.DummyRequest()
#        mgr = OrganizationManager(request)
#
#        new_org = mgr.get_by_pk(organization.pk)
#
#        assert organization == new_org
#
#    def test_get_organization_bad_pk(self):
#        from horus.models import Organization
#        from horus.models import User
#        from horus.managers import OrganizationManager
#
#        user = User(username='sontek', password='temp')
#        self.session.add(user)
#
#        organization = Organization('test org', user)
#        self.session.add(organization)
#        self.session.commit()
#
#        request = testing.DummyRequest()
#        mgr = OrganizationManager(request)
#
#        new_org = mgr.get_by_pk(99999)
#
#        assert new_org == None
#
#    def test_get_organization_get_all(self):
#        from horus.models import Organization
#        from horus.models import User
#        from horus.managers import OrganizationManager
#
#        user = User(username='sontek', password='temp')
#        self.session.add(user)
#
#        organization = Organization('test org', user)
#        self.session.add(organization)
#        self.session.commit()
#
#        request = testing.DummyRequest()
#        mgr = OrganizationManager(request)
#
#        orgs = mgr.get_all()
#
#        assert len(orgs) == 1
#
#class TestUserGroupManager(UnitTestBase):
#    def test_get_all(self):
#        from horus.models import UserGroup
#        from horus.models import User
#        from horus.managers import UserGroupManager
#
#        user = User(username='sontek', password='temp')
#        self.session.add(user)
#
#        group = UserGroup('admin', 'group for admins')
#        group.users.append(user)
#        self.session.add(group)
#        self.session.commit()
#
#        request = testing.DummyRequest()
#        mgr = UserGroupManager(request)
#
#        groups = mgr.get_all()
#
#        assert len(groups) == 1
#
#    def test_get_by_pk(self):
#        from horus.models import UserGroup
#        from horus.models import User
#        from horus.managers import UserGroupManager
#
#        group = UserGroup('admin', 'group for admins')
#        group2 = UserGroup('employees', 'group for employees')
#
#        self.session.add(group)
#        self.session.add(group2)
#
#        self.session.commit()
#
#        request = testing.DummyRequest()
#        mgr = UserGroupManager(request)
#
#        group = mgr.get_by_pk(2)
#
#        assert group.name == 'employees'
#
#
#
#



from horus.schemas      import LoginSchema
from horus.schemas      import RegisterSchema
from horus.schemas      import ForgotPasswordSchema
from horus.schemas      import ResetPasswordSchema
from horus.schemas      import ProfileSchema
from horus.forms        import SubmitForm
from horus.resources    import RootFactory
from horus.interfaces   import IHorusUserClass
from horus.interfaces   import IHorusActivationClass
from horus.interfaces   import IHorusLoginForm
from horus.interfaces   import IHorusLoginSchema
from horus.interfaces   import IHorusRegisterForm
from horus.interfaces   import IHorusRegisterSchema
from horus.interfaces   import IHorusForgotPasswordForm
from horus.interfaces   import IHorusForgotPasswordSchema
from horus.interfaces   import IHorusResetPasswordForm
from horus.interfaces   import IHorusResetPasswordSchema
from horus.interfaces   import IHorusProfileForm
from horus.interfaces   import IHorusProfileSchema
from horus.routes       import build_routes
from horus.lib          import get_user
from horus.lib          import get_class_from_config

def groupfinder(userid, request):
    user = request.user
    groups = []

    if user:
        for group in user.groups:
            groups.append('group:%s' % group.name)

        groups.append('user:%s' % user.pk)

    return groups

def includeme(config):
    settings = config.registry.settings
    config.set_request_property(get_user, 'user', reify=True)

    config.set_root_factory(RootFactory)


    if not config.registry.queryUtility(IHorusUserClass):
        user_class = get_class_from_config(settings, 'horus.user_class')
        config.registry.registerUtility(user_class, IHorusUserClass)

    if not config.registry.queryUtility(IHorusUserClass):
        activation_class = get_class_from_config(settings,
                'horus.activation_class')
        config.registry.registerUtility(activation_class,
                IHorusActivationClass)


    schemas = [
        (IHorusLoginSchema, LoginSchema),
        (IHorusRegisterSchema, RegisterSchema),
        (IHorusForgotPasswordSchema, ForgotPasswordSchema),
        (IHorusResetPasswordSchema, ResetPasswordSchema),
        (IHorusProfileSchema, ProfileSchema)
    ]

    forms = [
        IHorusLoginForm, IHorusRegisterForm, IHorusForgotPasswordForm,
        IHorusResetPasswordForm, IHorusProfileForm
    ]

    for iface, schema in schemas:
        if not config.registry.queryUtility(iface):
            config.registry.registerUtility(schema, iface)

    for form in forms:
        if not config.registry.queryUtility(form):
            config.registry.registerUtility(SubmitForm, form)

    config.include(build_routes)
    config.scan('horus')



import deform

class SubmitForm(deform.Form):
    def __init__(self, *args, **kwargs):

        if not kwargs.get('buttons'):
            kwargs['buttons'] = ('submit', )

        super(SubmitForm, self).__init__(*args, **kwargs)




from horus.tests import UnitTestBase

class TestInterfaces(UnitTestBase):
    def test_susession(self):
        """ Shouldn't be able to instantiate the interface """
        from horus.interfaces import IHorusSession

        def make_session():
            IHorusSession('1')

        self.assertRaises(TypeError, make_session)

    def test_suloginschema(self):
        """ Shouldn't be able to instantiate the interface """
        from horus.interfaces import IHorusLoginSchema

        def make_session():
            IHorusLoginSchema('1')

        self.assertRaises(TypeError, make_session)

    def test_suloginform(self):
        """ Shouldn't be able to instantiate the interface """
        from horus.interfaces import IHorusLoginForm

        def make_session():
            IHorusLoginForm('1')

        self.assertRaises(TypeError, make_session)



import colander
import deform

from pyramid_deform import CSRFSchema

class LoginSchema(CSRFSchema):
    User_name = colander.SchemaNode(colander.String())
    Password = colander.SchemaNode(colander.String(),
        validator=colander.Length(min=2),
        widget=deform.widget.PasswordWidget())

class RegisterSchema(CSRFSchema):
    User_name = colander.SchemaNode(colander.String())
    Email = colander.SchemaNode(colander.String(),
        validator=colander.Email())
    Password = colander.SchemaNode(colander.String(),
        validator=colander.Length(min=2),
        widget=deform.widget.CheckedPasswordWidget())

class ForgotPasswordSchema(CSRFSchema):
    Email = colander.SchemaNode(colander.String(),
        validator=colander.Email())

class ResetPasswordSchema(CSRFSchema):
    User_name = colander.SchemaNode(colander.String(),
            widget=deform.widget.TextInputWidget(template='readonly/textinput'),
            missing=colander.null,
    )
    Password = colander.SchemaNode(colander.String(),
        validator=colander.Length(min=2),
        widget=deform.widget.CheckedPasswordWidget())

class ProfileSchema(CSRFSchema):
    User_name = colander.SchemaNode(colander.String(),
            widget=deform.widget.TextInputWidget(template='readonly/textinput'),
            missing=colander.null,
        )
    Email = colander.SchemaNode(colander.String(),
        validator=colander.Email())
    Password = colander.SchemaNode(colander.String(),
        validator=colander.Length(min=2),
        widget=deform.widget.CheckedPasswordWidget(),
        missing=colander.null)



import os

from setuptools import setup, find_packages

here = os.path.abspath(os.path.dirname(__file__))
README = ''#open(os.path.join(here, 'README.md')).read()
CHANGES = ''#open(os.path.join(here, 'CHANGES.txt')).read()

requires = [
    'pyramid'
    , 'sqlalchemy'
    , 'cryptacular'
    , 'colander'
    , 'deform'
    , 'pyramid_deform'
    , 'transaction'
    , 'zope.sqlalchemy'
    , 'pystache'
    , 'pyramid_mailer'
    , 'beaker'
    , 'pyramid_beaker'
]

setup(name='horus'
      , version='0.8'
      , description='Generic user registration for pyramid'
      , long_description=README + '\n\n' +  CHANGES
      , classifiers=[
            'Intended Audience :: Developers'
            , 'License :: OSI Approved :: BSD License'
            , 'Operating System :: OS Independent'
            , 'Programming Language :: Python'
            , 'Topic :: Internet :: WWW/HTTP :: Dynamic Content'
            , 'Topic :: Software Development :: Libraries :: Python Modules'
      ]
      , author='John Anderson'
      , author_email='sontek@gmail.com'
      , url='https://github.com/sontek/horus'
      , keywords='pyramid'
      , license='BSD'
      , packages=find_packages()
      , include_package_data=True
      , zip_safe=False
      , install_requires=requires
      , tests_require=requires + ['pytest', 'mock', 'webtest', 'pytest-cov']
      , test_suite='horus'
)




from sqlalchemy.ext.declarative import declarative_base
from horus.models               import GroupMixin
from horus.models               import UserMixin
from horus.models               import UserGroupMixin
from horus.models               import ActivationMixin
from horus.models               import BaseModel

Base = declarative_base(cls=BaseModel)

class User(UserMixin, Base):
    pass

class Group(GroupMixin, Base):
    pass

class UserGroup(UserGroupMixin, Base):
    pass

class Activation(ActivationMixin, Base):
    pass



from horus.tests import UnitTestBase
from horus.tests.models import Base
from sqlalchemy.types import DateTime

from sqlalchemy import Column

from datetime import datetime

class TestModel(Base):
    start_date = Column(DateTime)

class TestModels(UnitTestBase):
    def test_tablename(self):
        model = TestModel()
        assert model.__tablename__ == 'test_model'

    def test_json(self):
        model = TestModel()
        model.pk = 1
        model.start_date = datetime.now()

        assert model.__json__() == {'pk': 1, 'start_date': model.start_date.isoformat()}

class TestActivation(UnitTestBase):
    def test_create_activation_without_valid_until(self):
        from horus.tests.models import Activation

        activation1 = Activation()

        assert activation1.code != None
        assert activation1.valid_until > datetime.utcnow()

    def test_create_activation_with_valid_until(self):
        from horus.tests.models import Activation

        dt = datetime.utcnow()
        activation1 = Activation(valid_until=dt)

        assert activation1.code != None
        assert activation1.valid_until == dt

class TestUser(UnitTestBase):
    def test_password_hashing(self):
        from horus.tests.models import User
        user1 = User(user_name='sontek', email='sontek@gmail.com')
        user1.set_password('password')
        self.session.add(user1)
        self.session.flush()

        assert user1.password != 'password'
        assert user1.salt != None

    def test_acl(self):
        from horus.tests.models import User
        from pyramid.security import Allow

        user1 = User(user_name='sontek', email='sontek@gmail.com')
        user1.set_password('foo')

        self.session.add(user1)
        self.session.flush()

        assert user1.__acl__ == [(Allow, 'user:%s' % user1.pk, 'access_user')]


class TestGroup(UnitTestBase):
    def test_init(self):
        from horus.tests.models import Group
        group = Group(name='foo', description='bar')

        assert group.name == 'foo'
        assert group.description == 'bar'
